from pathlib import Path

import cv2

from weedcv.weedcv import params
from weedcv.weedcv._debug import _debug


def otsu_thresh(mask, method_name="_otsu_thresh_", kernel_size=(3, 3)):
    """
    Denoises mask and/or grayscale image by first applying
    a gaussian filter then using Otsu's thresholding.

    Inputs:
    mask        = gray scale image (uint8 single-channel np array)

    Outputs:
    mask_th = binary image (np array)
    """

    mask_blur = cv2.GaussianBlur(mask, kernel_size, 0).astype("uint8")
    adaptive_method = cv2.THRESH_BINARY + cv2.THRESH_OTSU

    ret, bin_img = cv2.threshold(mask_blur, 0, 255, adaptive_method)

    # Threshold the image
    _debug(
        bin_img,
        filename=Path(
            params.debug_outdir,
            str(params.debug_filename) + "_" + str(params.device) + "_otsu.PNG",
        ),
    )

    return bin_img



import cv2

from weedcv.weedcv import params


def show_images(visual, filename):
    window = "results"
    visual = cv2.resize(visual, (600, 400))
    cv2.imshow(window, visual)
    cv2.waitKey(0)
    cv2.destroyAllWindows()



from pathlib import Path

import cv2
import numpy as np


def save_image(visual, filename):
    """
    Saves annotation results to a file for review
    """
    image_type = type(visual)
    if image_type == np.ndarray:
        cv2.imwrite(str(filename), visual)



# Debugging module
from pathlib import Path
from re import search

import cv2

from weedcv.weedcv import params, save_image
from weedcv.weedcv.show_image import show_images


def _debug(visual, filename=None):
    """
    Adapted from plantcv
    https://github.com/danforthcenter/plantcv/blob/master/plantcv/plantcv/_debug.py

    Save or display a visual for debugging.
    Inputs:
    visual   - An image or plot to display for debugging
    save_path - An optional filename to save the visual to (default: None)
    :param visual: numpy.ndarray
    :param filename: str
    """
    # Auto-increment the device counter
    params.device += 1

    if params.debug.lower() == "show":
        # show the image using opencv imshow
        show_images(visual=visual, filename=filename)

    elif params.debug.lower() == "save":
        Path(filename).parent.mkdir(parents=True, exist_ok=True)
        # save results to "save" directory
        save_image(visual=visual, filename=filename)

    elif params.debug.lower() == "input_image":
        if search(params.debug, filename.parts[-1]):
            Path(filename).parent.mkdir(parents=True, exist_ok=True)
            save_image(visual=visual, filename=filename)

        else:
            pass

    elif params.debug.lower() == "exg":
        if search(params.debug, filename.parts[-1]):
            Path(filename).parent.mkdir(parents=True, exist_ok=True)
            save_image(visual=visual, filename=filename)
        else:
            pass

    elif params.debug.lower() == "otsu":
        if search(params.debug, filename.parts[-1]):
            Path(filename).parent.mkdir(parents=True, exist_ok=True)
            save_image(visual=visual, filename=filename)
        else:
            pass
    elif params.debug.lower() == "mask":
        if search(params.debug, filename.parts[-1]):
            Path(filename).parent.mkdir(parents=True, exist_ok=True)
            save_image(visual=visual, filename=filename)
        else:
            pass

    elif params.debug.lower() == "combined_mask":
        if search(params.debug, filename.parts[-1]):
            Path(filename).parent.mkdir(parents=True, exist_ok=True)
            save_image(visual=visual, filename=filename)
        else:
            pass

    elif params.debug.lower() == "foreground":
        if search(params.debug, filename.parts[-1]):
            Path(filename).parent.mkdir(parents=True, exist_ok=True)
            save_image(visual=visual, filename=filename)
        else:
            pass

    elif params.debug is None:
        pass


#     elif params.debug == "debug exg":
# window = params.debug
# visual = cv2.resize(visual, (600, 400))
# cv2.imshow(window, visual)
# cv2.waitKey(0)
# cv2.destroyAllWindows()
#         # If debug is print, save the image to a file
# print_image(visual=visual, save_path=save_path)



from pathlib import Path

import cv2
import numpy as np

from weedcv.weedcv import params
from weedcv.weedcv._debug import _debug


def create_foreground(img, mask):
    # applys mask to create RGBA foreground using np
    mask = np.dstack([mask] * 3)  # Create 3-channel alpha mask
    mask = mask.astype("float32") / 255.0  # Use float matrices,
    img = img.astype("float32") / 255.0  # for easy blending

    masked = (mask * img) + ((1 - mask) * 255)  # Blend
    masked = (masked * 255).astype("uint8")

    _debug(
        masked,
        filename=Path(
            params.debug_outdir,
            params.debug_filename + "_" + str(params.device) + "_foreground.PNG",
        ),
    )
    return masked



# import os
# import unittest
# from pathlib import Path

# import cv2
# from weedcv.classes import Params as params
# from weedcv.weedcv.clear_edges import clear_edges
# from weedcv.weedcv.load_img import load_img
# from weedcv.weedcv.make_exg import make_exg
# from weedcv.weedcv.otsu_thresh import otsu_thresh

# from weedcv.weedcv.morph_close import morph_close
# from weedcv.weedcv.filter_components import filter_components

# TEST_DATA = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
# TEST_INPUT_EXAMPLE = str(Path(TEST_DATA, "palmer_residue_overhead_001.JPG"))
# TEST_EXAMPLE_DIM = (6000, 4000, 3)


# class Test(unittest.TestCase):
# def test_load_img(self):
# params.debug = None
# img, path = load_img(TEST_INPUT_EXAMPLE, mode="rgb")
# if img is None:
# raise AssertionError("Image is None")
# elif not Path(path).is_file():
# raise AssertionError("File does not exist: %s" % str(path))

# def test_make_exg(self):
# params.debug = None
# img = cv2.imread(TEST_INPUT_EXAMPLE)
# rgb_img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
# exg = make_exg(rgb_img, exg_thresh=0)
# if exg is None:
# raise AssertionError("ExG is None")

# def test_otsu_thresh(self):
# params.debug = None
# img = cv2.imread(TEST_INPUT_EXAMPLE)
# rgb_img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
# exg = make_exg(rgb_img, exg_thresh=0)
# otsu = otsu_thresh(exg)
# if otsu is None:
# raise AssertionError("OTSU is None")

# def test_clear_edges(self):
# params.debug = None
# img = cv2.imread(TEST_INPUT_EXAMPLE)
# rgb_img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
# exg = make_exg(rgb_img, exg_thresh=0)
# otsu = otsu_thresh(exg)
# res = clear_edges(otsu)
# if res is None:
# raise AssertionError("Clear_edges result is None")

# def test_morph_close(self):
#     params.debug = None
#     img = cv2.imread(TEST_INPUT_EXAMPLE)
#     rgb_img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
#     exg = make_exg(rgb_img, exg_thresh=0)
#     otsu = otsu_thresh(exg)
#     res = clear_edges(otsu)
#     morph = morph_close(res, 500, 500)
#     if morph is None:
#         raise AssertionError("Morph_close result is None")

# def test_filter_components(self):
#     params.debug = None
#     img = cv2.imread(TEST_INPUT_EXAMPLE)
#     rgb_img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
#     exg = make_exg(rgb_img, exg_thresh=0)
#     otsu = otsu_thresh(exg)
#     res = clear_edges(otsu)
#     morph = morph_close(res, 500, 500)
#     filtered = filter_components(morp, 1)
#     if filtered is None:
#         raise AssertionError("Filtered result is None")


# if __name__ == "__main__":
# unittest.main()



# Code wcas adapted fromplantcv:
# https://github.com/danforthcenter/plantcv/blob/master/plantcv/plantcv/watershed.py


import cv2
import numpy as np
import scipy
from scipy import ndimage as ndi
from skimage.feature import peak_local_max
from skimage.segmentation import watershed

from weedcv.weedcv import apply_mask, color_palette, params


def watershed_segmentation(rgb_img, mask, distance=10, label="default"):
    """Uses the watershed algorithm to detect boundary of objects. Needs a marker file which specifies area which is
       object (white), background (grey), unknown area (black).
    Inputs:
    rgb_img             = image to perform watershed on needs to be 3D (i.e. np.shape = x,y,z not np.shape = x,y)
    mask                = binary image, single channel, object in white and background black
    distance            = min_distance of local maximum
    label               = optional label parameter, modifies the variable name of observations recorded
    Returns:
    analysis_images     = list of output images
    :param rgb_img: numpy.ndarray
    :param mask: numpy.ndarray
    :param distance: int
    :param label: str
    :return analysis_images: list
    """
    # params.device += 1

    # Store debug mode
    # debug = params.debug
    # params.debug = None

    dist_transform = cv2.distanceTransformWithLabels(mask, cv2.DIST_L2, maskSize=0)[0]

    local_max = peak_local_max(
        dist_transform, indices=False, min_distance=distance, labels=mask
    )

    markers = ndi.label(local_max, structure=np.ones((3, 3)))[0]
    dist_transform1 = -dist_transform
    labels = watershed(dist_transform1, markers, mask=mask)

    img1 = np.copy(rgb_img)

    for x in np.unique(labels):
        rand_color = color_palette(len(np.unique(labels)))
        img1[labels == x] = rand_color[x]

    img2 = apply_mask(img1, mask, "black")

    joined = np.concatenate((img2, rgb_img), axis=1)

    # estimated_object_count = len(np.unique(markers)) - 1

    # # Reset debug mode
    # params.debug = debug
    # if params.debug == "print":
    # print_image(
    # dist_transform,
    # os.path.join(
    # params.debug_outdir, str(params.device) + "_watershed_dist_img.png"
    # ),
    # )
    # print_image(
    # joined,
    # os.path.join(
    # params.debug_outdir, str(params.device) + "_watershed_img.png"
    # ),
    # )
    # elif params.debug == "plot":
    # plot_image(dist_transform, cmap="gray")
    # plot_image(joined)

    # outputs.add_observation(
    # sample=label,
    # variable="estimated_object_count",
    # trait="estimated object count",
    # method="plantcv.plantcv.watershed",
    # scale="none",
    # datatype=int,
    # value=estimated_object_count,
    # label="none",
    # )

    # # Store images
    # outputs.images.append([dist_transform, joined])

    return joined, img2



"""
Reporter adapted from:
https://github.com/navarasu/opencv-log/blob/master/cvtest/reporter.py
"""

import os
import traceback
from pathlib import Path
from uuid import uuid4

import cv2
from cvlog import log

from weedcv.weedcv import params
from weedcv.weedcv.csv_reporter import CsvReporter


class Reporter:
    def __new__(cls):
        if not hasattr(cls, "instance") or not cls.instance:
            cls.instance = super().__new__(cls)
            cls.instance.__initialised = False
        return cls.instance

    def __init__(self):
        if not self.__initialised:
            self.__initialised = True
            report_path = Path(params.log_path) / "report"
            self.image_path = report_path / "images"
            self.reporter = CsvReporter(str(report_path / "report.csv"))

    def result(self, input_image, key_pressed, output_img):
        message = ""
        if key_pressed == ord("y"):
            result = "Pass"
        else:
            result = "Fail"
            message = self.__save_image(output_img)
        self.reporter.log_report([input_image, result.upper(), message])

    def __save_image(self, img):
        self.image_path.mkdir(parents=True, exist_ok=True)
        output_path = str(self.image_path / str(uuid4())) + ".png"
        cv2.imwrite(output_path, img)
        return output_path

    def error(self, input_image, ex):
        self.reporter.log_report([input_image, "ERROR", self.__stack_trace(ex)])

    def __stack_trace(self, ex):
        stacks = traceback.extract_tb(ex.__traceback__)[1:]
        stack_trace = ""
        for x in stacks[:10]:
            stack_trace += x.filename + ":" + str(x.lineno) + ";"
        return stack_trace


# def report(input_image_path, processing_method):
def report(input_image_path, processing_method):

    for image_path in input_image_path:
        results = processing_method(str(image_path))
        if isinstance(results, list):
            for proc_result in results:
                key_pressed = log.show_image(
                    str(image_path), "log_type", proc_result, None
                )
                Reporter().result(image_path, key_pressed, proc_result)
        else:
            key_pressed = log.show_image(str(image_path), "log_type", results, None)

            Reporter().result(image_path, key_pressed, results)



from pathlib import Path

import cv2

from weedcv.weedcv import params
from weedcv.weedcv._debug import _debug

"""
Adapted from Plantcv:
https://github.com/danforthcenter/plantcv/blob/master/plantcv/plantcv/readimage.py
"""


def load_img(path, mode="rgb"):
    """
    Read image from absolute image path.
    Inputs:
    path    = absolute image file path
    mode    = image format for output ("rgb", "bgr", "rgba", "gray")

    Returns:
    img     = image as numpy array
    path    = input path
    """
    if not Path(path).is_file():
        raise RuntimeError("Image path does not exists.")

    if mode.upper() == "RGB":
        img = cv2.imread(path)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    elif mode.upper() == "BGR":
        img = cv2.imread(path)
    elif mode.upper() == "RGBA":
        img = cv2.imread(path, -1)
    elif mode.upper() == "GRAY" or mode.upper() == "GREY":
        img = cv2.imread(path, 0)
    if img is None:
        raise RuntimeError("Failed to open " + path)

    _debug(
        img,
        filename=Path(
            params.debug_outdir,
            str(params.debug_filename) + "_" + str(params.device) + "_input_image.PNG",
        ),
    )

    return img, path



"""Adapted from
https://github.com/navarasu/opencv-log/blob/master/cvtest/csv_reporter.py
"""

import csv
from pathlib import Path


class CsvReporter:
    def __init__(self, file_path):
        self.__file_path = file_path
        self.__fields = ["Test Image", "Result", "Message"]
        self.__create_file()

    def __create_file(self):
        # Make directory
        path = Path(self.__file_path)
        dir_path = path.parents[0]
        dir_path.mkdir(parents=True, exist_ok=True)
        # Make file
        with open(self.__file_path, "w") as f:
            writer = csv.writer(f, lineterminator="\n")
            writer.writerow(self.__fields)

    def __append(self, values):
        with open(self.__file_path, "a") as f:
            writer = csv.writer(f, lineterminator="\n")
            writer.writerow(values)

    def log_report(self, result):
        self.__append(result)



from pathlib import Path

import skimage.morphology as morphology
from skimage.morphology import black_tophat, disk, opening, white_tophat

from weedcv.weedcv import otsu_thresh, params
from weedcv.weedcv._debug import _debug


def morph_close(mask, min_object_size, area_threshold):
    mask = morphology.remove_small_holes(
        morphology.remove_small_objects(mask.astype("uint8"), min_object_size),
        area_threshold,
    )

    # Close using white tophat
    w_disk = disk(4)
    # Close using black tophat
    b_disk = disk(2)
    # Tophat closing
    w_tophat = white_tophat(mask, w_disk)
    b_tophat = black_tophat(mask, b_disk)
    # Map results to mask
    mask[w_tophat == 255] = 255
    mask[b_tophat == 255] = 255
    # Otsu's thresh
    otsu_mask = otsu_thresh(mask.astype("uint8"))
    open_disk = disk(1)
    mask = opening(otsu_mask, open_disk)

    _debug(
        mask,
        filename=Path(
            params.debug_outdir,
            str(params.debug_filename) + "_" + str(params.device) + "_morph_close.PNG",
        ),
    )

    return mask



# get contours (presumably just one around the nonzero pixels)
import cv2


def crop_contours(mask):
    contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

    if len(contours) == 2:
        contours = contours[0]
    else:
        contours[1]

    cntr = contours[0]
    x, y, w, h = cv2.boundingRect(cntr)
    return x, y, w, h



# Apply White or Black Background Mask

import os

import cv2
import numpy as np

from weedcv.weedcv import params


def apply_mask(img, mask, mask_color):
    """Apply white image mask to image, with bitwise AND operator bitwise NOT operator and ADD operator.
    Inputs:
    img        = RGB image data
    mask       = Binary mask image data
    mask_color = 'white' or 'black'
    Returns:
    masked_img = masked image data
    :param img: numpy.ndarray
    :param mask: numpy.ndarray
    :param mask_color: str
    :return masked_img: numpy.ndarray
    """

    params.device += 1

    if mask_color.upper() == "WHITE":
        color_val = 255
    elif mask_color.upper() == "BLACK":
        color_val = 0

    array_data = img.copy()

    # Mask the array
    array_data[np.where(mask == 0)] = color_val

    return array_data



# from weedcv.weedcv import apply_mask
from pathlib import Path

import numpy as np

from weedcv.weedcv import params
from weedcv.weedcv._debug import _debug


def combine_component(component_list):
    input_img_shape = component_list[0].shape
    comb_mask = np.zeros(input_img_shape)
    for component in component_list:
        comb_mask[np.where(component != 0)] = 255

    _debug(
        comb_mask,
        filename=Path(
            params.debug_outdir,
            params.debug_filename + "_" + str(params.device) + "_combined_mask.PNG",
        ),
    )
    return comb_mask



import os


class Params:
    """PlantCV parameters class."""

    def __init__(
        self,
        device=0,
        debug=None,
        debug_filename="debug_results",
        debug_outdir="results/debug",
        line_thickness=5,
        log_path=".",
        dpi=100,
        text_size=0.55,
        text_thickness=2,
        marker_size=60,
        color_scale="gist_rainbow",
        color_sequence="sequential",
        saved_color_scale=None,
        verbose=True,
    ):
        """Initialize parameters.
        Keyword arguments/parameters:
        device            = Device number. Used to count steps in the pipeline. (default: 0)
        debug             = None, print, or plot. Print = save to file, Plot = print to screen. (default: None)
        debug_outdir      = Debug images output directory. (default: .)
        line_thickness    = Width of line drawings. (default: 5)
        dpi               = Figure plotting resolution, dots per inch. (default: 100)
        text_size         = Size of plotting text. (default: 0.55)
        text_thickness    = Thickness of plotting text. (default: 2)
        marker_size       = Size of plotting markers (default: 60)
        color_scale       = Name of plotting color scale (matplotlib colormap). (default: gist_rainbow)
        color_sequence    = Build color scales in "sequential" or "random" order. (default: sequential)
        saved_color_scale = Saved color scale that will be applied next time color_palette is called. (default: None)
        verbose           = Whether or not in verbose mode. (default: True)
        :param device: int
        :param debug: str "save"
        :param debug_outdir: str
        :param line_thickness: numeric
        :param dpi: int
        :param text_size: float
        :param text_thickness: int
        :param marker_size: int
        :param color_scale: str
        :param color_sequence: str
        :param saved_color_scale: list
        :param verbose: bool
        """

        self.device = device
        self.debug = debug
        self.debug_outdir = debug_outdir
        self.debug_filename = debug_filename
        self.line_thickness = line_thickness
        self.log_path = log_path
        self.dpi = dpi
        self.text_size = text_size
        self.text_thickness = text_thickness
        self.marker_size = marker_size
        self.color_scale = color_scale
        self.color_sequence = color_sequence
        self.saved_color_scale = saved_color_scale
        self.verbose = verbose



from pathlib import Path

import cv2
import numpy as np

from weedcv.weedcv import params
from weedcv.weedcv._debug import _debug


def filter_components(mask, top_n):
    # calculate size of individual components and chooses based on min size
    nb_components, output, stats, centroids = cv2.connectedComponentsWithStats(
        mask, connectivity=8
    )
    # size of components except 0 (background)
    sizes = stats[1:, -1]
    nb_components = nb_components - 1
    # Determines number of components to segment
    # Sort components from largest to smallest
    top_n_sizes = sorted(sizes, reverse=True)[:top_n]
    try:
        min_size = min(top_n_sizes) - 1
    except:
        min_size = 0
    list_filtered_masks = []
    for i in range(0, nb_components):
        if sizes[i] >= min_size:
            filtered_mask = np.zeros((output.shape))
            filtered_mask[output == i + 1] = 255
            list_filtered_masks.append(filtered_mask)

    for mask in list_filtered_masks:
        # Print or plot the binary image if debug is on
        _debug(
            mask,
            filename=Path(
                params.debug_outdir,
                str(params.debug_filename) + "_" + str(params.device) + ".PNG",
            ),
        )
    return list_filtered_masks



from pathlib import Path

import setuptools

# The directory containing this file
HERE = Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# requirements_f = open('requirements.txt', 'r')
# dependencies = [req for req in requirements_f.readlines()]

# This call to setup() does all the work
setuptools.setup(
    name="weedcv",  # how project shows up on website
    version="1.0.4",
    description="Image preprocessing and dataset development tools for weed detection.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/mkutu/weedcv",
    author="WeedCV Team",
    author_email="help@weedcv.com",
    license="MIT",
    classifiers=[
        # How mature is this project? Common values are
        #   2 - Pre-Alpha
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 2 - Pre-Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Image Processing",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
    ],
    # What does your project relate to?
    keywords="dataset development, weed detection, digital agriculture",
    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=setuptools.find_packages(),
    # package_dir={"": "weedcv"},
    # packages=setup.find_packages(where="weedcv"),
    install_requires=["numpy", "matplotlib", "opencv-python>4", "scikit-image>0.17.0"],
    test_suite="nose.collector",
    tests_require=["nose"],
)



import os
from pathlib import Path

from skimage.segmentation import clear_border

from weedcv.weedcv import params
from weedcv.weedcv._debug import _debug


def clear_edges(bin_img):
    """
    Denoising by removing vegetations that extend beyond image frame.
    Input:
    binary_img = np array

    Output:
    cleared_binary = np array
    """
    cleared_binary = clear_border(bin_img)

    # Print or plot the binary image if debug is on
    _debug(
        cleared_binary,
        filename=Path(
            params.debug_outdir,
            str(params.debug_filename) + "_" + str(params.device) + ".PNG",
        ),
    )

    return cleared_binary



"""
isort:skip_file
"""
# reporter = Reporter()

from weedcv.weedcv.classes import Params

from weedcv.weedcv.csv_reporter import CsvReporter

params = Params()


from weedcv.weedcv.save_image import save_image
from weedcv.weedcv.show_image import show_images
from weedcv.weedcv.apply_mask import apply_mask
from weedcv.weedcv.clear_edges import clear_edges
from weedcv.weedcv.color_palette import color_palette
from weedcv.weedcv.create_foreground import create_foreground
from weedcv.weedcv.crop_contours import crop_contours
from weedcv.weedcv.filter_components import filter_components
from weedcv.weedcv.load_img import load_img
from weedcv.weedcv.make_exg import make_exg
from weedcv.weedcv.morph_close import morph_close
from weedcv.weedcv.otsu_thresh import otsu_thresh
from weedcv.weedcv.watershed import watershed_segmentation
from weedcv.weedcv.combine_component import combine_component

# from .reporter import report

# __all__ = ["report"]
# from weedcv.weedcv.reporter import Reporter


__all__ = [
    "apply_mask",
    "clear_edges",
    "color_palette",
    "create_foreground",
    "crop_contours",
    "filter_components",
    "load_img",
    "make_exg",
    "morph_close",
    "otsu_thresh",
    "save_image",
    "combine_component",
    "show_images",
    "watershed_segmentation",
]



from pathlib import Path

import cv2
import numpy as np

from weedcv.weedcv import params
from weedcv.weedcv._debug import _debug


def make_exg(rgb_img, exg_thresh=0):
    """
    Excess Green Index

    r = R / (R + G + B)
    g = G / (R + G + B)
    b = B / (R + G + B)

    EXG = 2 * G - R - B

    ExG values range from -1 to 2.

    Inputs:
    rgb_img      = np array in RGB

    Returns:
    index_array    = single channel np array (float32)
    """

    if rgb_img is None:
        raise RuntimeError("RGB image is None.")

    img = rgb_img.astype(float)

    blue = img[:, :, 2]
    green = img[:, :, 1]
    red = img[:, :, 0]

    exg = (2 * green) - red - blue

    # Thresholding removes low negative values
    exg = np.where(exg < exg_thresh, 0, exg).astype("uint8")

    _debug(
        exg,
        filename=Path(
            params.debug_outdir,
            str(params.debug_filename) + "_" + str(params.device) + "_exg.PNG",
        ),
    )
    return exg
